package javax.swing.text.html;

import java.awt.Rectangle;
import java.util.HashMap;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Polygon;
import java.awt.Color;
import java.util.Map;
import javax.swing.text.AttributeSet;
import javax.swing.border.AbstractBorder;

class CSSBorder extends AbstractBorder
{
    static final int COLOR = 0;
    static final int STYLE = 1;
    static final int WIDTH = 2;
    static final int TOP = 0;
    static final int RIGHT = 1;
    static final int BOTTOM = 2;
    static final int LEFT = 3;
    static final CSS.Attribute[][] ATTRIBUTES;
    static final CSS.CssValue[] PARSERS;
    static final Object[] DEFAULTS;
    final AttributeSet attrs;
    static Map<CSS.Value, CSSBorder.BorderPainter> borderPainters;
    
    CSSBorder(AttributeSet attrs) {
        this.attrs = attrs;
    }
    
    private Color getBorderColor(int n) {
        Object attribute = this.attrs.getAttribute(CSSBorder.ATTRIBUTES[0][n]);
        CSS.ColorValue colorValue;
        if ((attribute instanceof CSS.ColorValue)) {
            colorValue = (CSS.ColorValue)attribute;
        }
        else {
            colorValue = (CSS.ColorValue)this.attrs.getAttribute(CSS.Attribute.COLOR);
            if (colorValue == null) {
                colorValue = (CSS.ColorValue)CSSBorder.PARSERS[0].parseCssValue(CSS.Attribute.COLOR.getDefaultValue());
            }
        }
        return colorValue.getValue();
    }
    
    private int getBorderWidth(int n) {
        int n2 = 0;
        CSS.BorderStyle borderStyle = (CSS.BorderStyle)this.attrs.getAttribute(CSSBorder.ATTRIBUTES[1][n]);
        if (borderStyle != null && borderStyle.getValue() != CSS.Value.NONE) {
            CSS.LengthValue lengthValue = (CSS.LengthValue)this.attrs.getAttribute(CSSBorder.ATTRIBUTES[2][n]);
            if (lengthValue == null) {
                lengthValue = (CSS.LengthValue)CSSBorder.DEFAULTS[2];
            }
            n2 = (int)lengthValue.getValue(true);
        }
        return n2;
    }
    
    private int[] getWidths() {
        int[] array = new int[4];
        for (int i = 0; i < array.length; ++i) {
            array[i] = this.getBorderWidth(i);
        }
        return array;
    }
    
    private CSS.Value getBorderStyle(int n) {
        CSS.BorderStyle borderStyle = (CSS.BorderStyle)this.attrs.getAttribute(CSSBorder.ATTRIBUTES[1][n]);
        if (borderStyle == null) {
            borderStyle = (CSS.BorderStyle)CSSBorder.DEFAULTS[1];
        }
        return borderStyle.getValue();
    }
    
    private Polygon getBorderShape(int n) {
        Polygon polygon = null;
        int[] widths = this.getWidths();
        if (widths[n] != 0) {
            polygon = new Polygon(new int[4], new int[4], 0);
            polygon.addPoint(0, 0);
            polygon.addPoint(-widths[(n + 3) % 4], -widths[n]);
            polygon.addPoint(widths[(n + 1) % 4], -widths[n]);
            polygon.addPoint(0, 0);
            return polygon;
        }
        return polygon;
    }
    
    private CSSBorder.BorderPainter getBorderPainter(int n) {
        return (CSSBorder.BorderPainter)CSSBorder.borderPainters.get(this.getBorderStyle(n));
    }
    
    static Color getAdjustedColor(Color color, double a) {
        double n = 1.0 - Math.min(Math.abs(a), 1.0);
        double n2 = (a > 0.0) ? (255.0 * (1.0 - n)) : 0.0;
        return new Color((int)((double)color.getRed() * n + n2), (int)((double)color.getGreen() * n + n2), (int)((double)color.getBlue() * n + n2));
    }
    
    @Override
    public Insets getBorderInsets(Component component, Insets insets) {
        int[] widths = this.getWidths();
        insets.set(widths[0], widths[3], widths[2], widths[1]);
        return insets;
    }
    
    @Override
    public void paintBorder(Component component, Graphics graphics, int n, int n2, int n3, int n4) {
        if (!(graphics instanceof Graphics2D)) {
            return;
        }
        Graphics2D graphics2D = (Graphics2D)graphics.create();
        int[] widths = this.getWidths();
        int n5 = n + widths[3];
        int n6 = n2 + widths[0];
        int n7 = n3 - (widths[1] + widths[3]);
        int n8 = n4 - (widths[0] + widths[2]);
        int[][] array = { { n5, n6 }, { n5 + n7, n6 }, { n5 + n7, n6 + n8 }, { n5, n6 + n8 } };
        for (int i = 0; i < 4; ++i) {
            CSS.Value borderStyle = this.getBorderStyle(i);
            Polygon borderShape = this.getBorderShape(i);
            if (borderStyle != CSS.Value.NONE && borderShape != null) {
                n8 = ((i % 2 == 0) ? n7 : n8);
                int[] xpoints = borderShape.xpoints;
                int n9 = 2;
                xpoints[n9] += n8;
                int[] xpoints2 = borderShape.xpoints;
                int n10 = 3;
                xpoints2[n10] += n8;
                Color borderColor = this.getBorderColor(i);
                CSSBorder.BorderPainter borderPainter = this.getBorderPainter(i);
                double n11 = (double)i * 3.141592653589793 / 2.0;
                graphics2D.setClip(graphics.getClip());
                graphics2D.translate(array[i][0], array[i][1]);
                graphics2D.rotate(n11);
                graphics2D.clip(borderShape);
                borderPainter.paint(borderShape, graphics2D, borderColor, i);
                graphics2D.rotate(-n11);
                graphics2D.translate(-array[i][0], -array[i][1]);
            }
        }
        graphics2D.dispose();
        return;
    }
    
    static void registerBorderPainter(CSS.Value value, CSSBorder.BorderPainter borderPainter) {
        CSSBorder.borderPainters.put(value, borderPainter);
    }
    
    static {
        ATTRIBUTES = new CSS.Attribute[][] { { CSS.Attribute.BORDER_TOP_COLOR, CSS.Attribute.BORDER_RIGHT_COLOR, CSS.Attribute.BORDER_BOTTOM_COLOR, CSS.Attribute.BORDER_LEFT_COLOR }, { CSS.Attribute.BORDER_TOP_STYLE, CSS.Attribute.BORDER_RIGHT_STYLE, CSS.Attribute.BORDER_BOTTOM_STYLE, CSS.Attribute.BORDER_LEFT_STYLE }, { CSS.Attribute.BORDER_TOP_WIDTH, CSS.Attribute.BORDER_RIGHT_WIDTH, CSS.Attribute.BORDER_BOTTOM_WIDTH, CSS.Attribute.BORDER_LEFT_WIDTH } };
        PARSERS = new CSS.CssValue[] { new CSS.ColorValue(), new CSS.BorderStyle(), new CSS.BorderWidthValue(null, 0) };
        DEFAULTS = new Object[] { CSS.Attribute.BORDER_COLOR, CSSBorder.PARSERS[1].parseCssValue(CSS.Attribute.BORDER_STYLE.getDefaultValue()), CSSBorder.PARSERS[2].parseCssValue(CSS.Attribute.BORDER_WIDTH.getDefaultValue()) };
        CSSBorder.borderPainters = new HashMap();
        registerBorderPainter(CSS.Value.NONE, new CSSBorder.NullPainter());
        registerBorderPainter(CSS.Value.HIDDEN, new CSSBorder.NullPainter());
        registerBorderPainter(CSS.Value.SOLID, new CSSBorder.SolidPainter());
        registerBorderPainter(CSS.Value.DOUBLE, new CSSBorder.DoublePainter());
        registerBorderPainter(CSS.Value.DOTTED, new CSSBorder.DottedDashedPainter(1));
        registerBorderPainter(CSS.Value.DASHED, new CSSBorder.DottedDashedPainter(3));
        registerBorderPainter(CSS.Value.GROOVE, new CSSBorder.GrooveRidgePainter(CSS.Value.GROOVE));
        registerBorderPainter(CSS.Value.RIDGE, new CSSBorder.GrooveRidgePainter(CSS.Value.RIDGE));
        registerBorderPainter(CSS.Value.INSET, new CSSBorder.InsetOutsetPainter(CSS.Value.INSET));
        registerBorderPainter(CSS.Value.OUTSET, new CSSBorder.InsetOutsetPainter(CSS.Value.OUTSET));
    }
    
    static class NullPainter implements CSSBorder.BorderPainter
    {
        @Override
        public void paint(Polygon polygon, Graphics graphics, Color color, int n) {
        }
    }
    
    static class SolidPainter implements CSSBorder.BorderPainter
    {
        @Override
        public void paint(Polygon p4, Graphics graphics, Color color, int n) {
            graphics.setColor(color);
            graphics.fillPolygon(p4);
        }
    }
    
    abstract static class StrokePainter implements CSSBorder.BorderPainter
    {
        void paintStrokes(Rectangle rectangle, Graphics graphics, int n, int[] array, Color[] array2) {
            int n2 = n == 0;
            int i = 0;
            int n3 = n2 ? rectangle.width : rectangle.height;
            while (i < n3) {
                int n5;
                for (int n4 = 0; n4 < array.length && i < n3; i += n5, ++n4) {
                    n5 = array[n4];
                    Color color = array2[n4];
                    if (color != null) {
                        int n6 = rectangle.x + (n2 ? i : 0);
                        int n7 = rectangle.y + (n2 ? 0 : i);
                        int n8 = n2 ? n5 : rectangle.width;
                        n5 = (n2 ? rectangle.height : n5);
                        graphics.setColor(color);
                        graphics.fillRect(n6, n7, n8, n5);
                    }
                }
            }
        }
    }
    
    static class DoublePainter extends CSSBorder.StrokePainter
    {
        @Override
        public void paint(Polygon polygon, Graphics graphics, Color color, int n) {
            Rectangle bounds = polygon.getBounds();
            int max = Math.max(bounds.height / 3, 1);
            this.paintStrokes(bounds, graphics, 1, new int[] { max, max }, new Color[] { color, null });
        }
    }
    
    static class DottedDashedPainter extends CSSBorder.StrokePainter
    {
        final int factor;
        
        DottedDashedPainter(int factor) {
            this.factor = factor;
        }
        
        @Override
        public void paint(Polygon polygon, Graphics graphics, Color color, int n) {
            Rectangle bounds = polygon.getBounds();
            int n2 = bounds.height * this.factor;
            this.paintStrokes(bounds, graphics, 0, new int[] { n2, n2 }, new Color[] { color, null });
        }
    }
    
    abstract static class ShadowLightPainter extends CSSBorder.StrokePainter
    {
        static Color getShadowColor(Color color) {
            return CSSBorder.getAdjustedColor(color, -0.3);
        }
        
        static Color getLightColor(Color color) {
            return CSSBorder.getAdjustedColor(color, 0.7);
        }
    }
    
    static class GrooveRidgePainter extends CSSBorder.ShadowLightPainter
    {
        final CSS.Value type;
        
        GrooveRidgePainter(CSS.Value type) {
            this.type = type;
        }
        
        @Override
        public void paint(Polygon polygon, Graphics graphics, Color color, int n) {
            Rectangle bounds = polygon.getBounds();
            int max = Math.max(bounds.height / 2, 1);
            this.paintStrokes(bounds, graphics, 1, new int[] { max, max }, ((n + 1) % 4 < 2 == (this.type == CSS.Value.GROOVE)) ? new Color[] { CSSBorder.ShadowLightPainter.getShadowColor(color), CSSBorder.ShadowLightPainter.getLightColor(color) } : new Color[] { CSSBorder.ShadowLightPainter.getLightColor(color), CSSBorder.ShadowLightPainter.getShadowColor(color) });
        }
    }
    
    static class InsetOutsetPainter extends CSSBorder.ShadowLightPainter
    {
        CSS.Value type;
        
        InsetOutsetPainter(CSS.Value type) {
            this.type = type;
        }
        
        @Override
        public void paint(Polygon p4, Graphics graphics, Color color, int n) {
            graphics.setColor(((n + 1) % 4 < 2 == (this.type == CSS.Value.INSET)) ? CSSBorder.ShadowLightPainter.getShadowColor(color) : CSSBorder.ShadowLightPainter.getLightColor(color));
            graphics.fillPolygon(p4);
        }
    }
    
    interface BorderPainter
    {
        public abstract void paint(Polygon p0, Graphics p1, Color p2, int p3);
    }
}
